@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -49,6 +49,7 @@
 #include "mpool.h"
 #define UNZIP_PRIVATE
 #include "unzip.h"
+#include "is_tar.h"
 
 // clang-format off
 static const struct ftmap_s {
@@ -86,6 +87,8 @@ static const struct ftmap_s {
     { "CL_TYPE_GRAPHICS",     CL_TYPE_GRAPHICS     },
     { "CL_TYPE_GIF",          CL_TYPE_GIF          },
     { "CL_TYPE_PNG",          CL_TYPE_PNG          },
+    { "CL_TYPE_JPEG",         CL_TYPE_JPEG         },
+    { "CL_TYPE_TIFF",         CL_TYPE_TIFF         },
     { "CL_TYPE_RIFF",         CL_TYPE_RIFF         },
     { "CL_TYPE_BINHEX",       CL_TYPE_BINHEX       },
     { "CL_TYPE_TNEF",         CL_TYPE_TNEF         },
@@ -217,8 +220,6 @@ cli_file_t cli_compare_ftm_file(const unsigned char *buf, size_t buflen, const s
     return cli_texttype(buf, buflen);
 }
 
-int is_tar(const unsigned char *buf, unsigned int nbytes);
-
 /* organize by length, cannot exceed SIZEOF_LOCAL_HEADER */
 // clang-format off
 const struct ooxml_ftcodes {
