@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2020 Cisco and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2022 Cisco and/or its affiliates. All rights reserved.
  *
  *  Author: Shawn Webb
  *
@@ -20,8 +20,11 @@
 
 #include <string.h>
 
+// libclamav
 #include "others.h"
 
+#include "output.h"
+
 char hostid[37];
 
 int is_valid_hostid(void)
